# general

class APITesterException(Exception):
    pass


# get method

class APITesterGetException(APITesterException):
    pass


class APITesterGetDataException(APITesterGetException):
    pass


class APITesterGetJsonException(APITesterGetException):
    pass


# put method
class APITesterPutException(APITesterException):
    pass


# delete method
class APITesterDeleteException(APITesterException):
    pass


# put method
class APITesterPostException(APITesterException):
    pass


# head method
class APITesterHeadException(APITesterException):
    pass


# options method
class APITesterOptionsException(APITesterException):
    pass


# patch method
class APITesterPatchException(APITesterException):
    pass


# session method


class APITesterSessionException(APITesterException):
    pass


# execute


class APITesterExecuteException(APITesterException):
    pass


# json
class APITesterJsonException(APITesterException):
    pass


# xml
class APITesterXMLException(APITesterException):
    pass


class APITesterXMLTypeException(APITesterXMLException):
    pass


# API data check
class APICheckException(APITesterException):
    pass


class APIStatusCodeException(APICheckException):
    pass


class APITextException(APICheckException):
    pass


class APIContentException(APICheckException):
    pass


class APIHeadersException(APICheckException):
    pass


class APIHistoryException(APICheckException):
    pass


class APIEncodingException(APICheckException):
    pass


class APICookiesException(APICheckException):
    pass


class APIElapsedException(APICheckException):
    pass


class APIRequestsMethodException(APICheckException):
    pass


class APIRequestsUrlException(APICheckException):
    pass


class APIRequestsBodyException(APICheckException):
    pass


# Assert exception

class APIAssertException(APITesterException):
    pass


# html exception

class APIHTMLException(APITesterException):
    pass


# Json

class APIJsonReportException(APITesterException):
    pass


# Executor

class APIAddCommandException(APITesterException):
    pass


# Execute callback
class CallbackExecutorException(APITesterException):
    pass



# Unknown error
error_message: str = "API tester error"
wrong_http_method_error_message: str = "wrong http method"
http_method_have_wrong_type: str = "get wrong type param"
# get data error
get_data_error_message: str = "API tester can't get api data error"
# Http Method error
get_error_message: str = "API tester get error"
get_json_error_message: str = "API tester get json error"
put_error_message: str = "API tester put error"
delete_error_message: str = "API tester delete error"
post_error_message: str = "API tester post error"
head_error_message: str = "API tester head error"
options_error_message: str = "API tester options error"
patch_error_message: str = "API tester patch error"
session_error_message: str = "API tester session error"
# response code error
response_code_error: str = "API tester get wrong response code"
# response body error
response_body_error: str = "API tester get wrong response body"
# response timeout error
response_timeout_error: str = "API tester timeout"
# response history error
response_history_error: str = "API tester history error"
# response encoding error
response_encoding_error: str = "API tester encoding error"
# response cookies error
response_cookies_error: str = "API tester cookies error"
# response headers error
response_headers_error: str = "API tester headers error"
# response content error
response_content_error: str = "API tester content error"
# response text error
response_text_error: str = "API tester text error"
# execute action error
execute_action_error: str = "can't execute action"
# json error
cant_find_json_error: str = "can't find json"
cant_save_json_error: str = "can't save json"
cant_save_json_report_record_us_null: str = "can't save json record is null"
cant_reformat_json_error: str = "can't reformat json is type right?"
cant_find_element_in_json_error: str = "can't find element in json"
json_type_error: str = "json type error"
wrong_json_data_error: str = "can't parser json"
# XML error
cant_read_xml_error: str = "can't read xml"
xml_type_error: str = "xml type error"
# Executor error
executor_data_error: str = "executor receive wrong data"
executor_list_error: str = "executor receive wrong data list is none or wrong type"
# API data check
status_code_error: str = "request status code error"
status_text_error: str = "request text error"
status_content_error: str = "request content error"
status_headers_error: str = "request headers error"
status_history_error: str = "request history error"
status_encoding_error: str = "request encoding error"
status_cookies_error: str = "request cookies error"
status_elapsed_error: str = "request elapsed error"
status_requests_method_error: str = "requests method error"
status_requests_url_error: str = "requests url error"
status_requests_body_error: str = "requests body error"
# HTML
html_generate_no_data_tag: str = "record is None"
# add command
add_command_exception_tag: str = "command value type should be as method or function"
# argparse
argparse_get_wrong_data: str = "argparse receive wrong data"
# Callback executor
get_bad_trigger_method: str = "get bad trigger method, only accept kwargs and args"
get_bad_trigger_function: str = "get bad trigger function only accept function in event_dict"



import json
import sys
from threading import Lock

from je_api_testka.utils.test_record.test_record_class import test_record_instance
from je_api_testka.utils.exception.exception_tags import cant_save_json_report_record_us_null
from je_api_testka.utils.exception.exceptions import APIJsonReportException


def generate_json():
    """
    :return: test success_dict test failure_dict
    """
    if len(test_record_instance.test_record_list) == 0 and len(test_record_instance.error_record_list) == 0:
        raise APIJsonReportException(cant_save_json_report_record_us_null)
    else:
        success_dict = dict()
        success_count: int = 1
        success_test_str: str = "Success_Test"
        for record_data in test_record_instance.test_record_list:
            success_dict.update(
                {
                    success_test_str + str(success_count): {
                        "status_code": str(record_data.get("status_code")),
                        "text": str(record_data.get("text")),
                        "content": str(record_data.get("content"), encoding="utf-8"),
                        "headers": str(record_data.get("headers")),
                        "history": str(record_data.get("history")),
                        "encoding": str(record_data.get("encoding")),
                        "cookies": str(record_data.get("cookies")),
                        "elapsed": str(record_data.get("elapsed")),
                        "request_time_sec": str(record_data.get("request_time_sec")),
                        "request_method": str(record_data.get("request_method")),
                        "request_url": str(record_data.get("request_url")),
                        "request_body": str(record_data.get("request_body")),
                        "start_time": str(record_data.get("start_time")),
                        "end_time": str(record_data.get("end_time")),
                    }
                }
            )
            success_count = success_count + 1
        failure_dict = dict()
        if len(test_record_instance.error_record_list) == 0:
            pass
        else:
            failure_count: int = 1
            failure_test_str: str = "Failure_Test"
            for record_data in test_record_instance.error_record_list:
                failure_dict.update(
                    {
                        failure_test_str: {
                            "http_method": str(record_data[0].get("http_method")),
                            "test_url": str(record_data[0].get("test_url")),
                            "soap": str(record_data[0].get("soap")),
                            "record_request_info": str(record_data[0].get("record_request_info")),
                            "clean_record": str(record_data[0].get("clean_record")),
                            "result_check_dict": str(record_data[0].get("result_check_dict")),
                            "error": str(record_data[1])
                        }
                    }
                )
                failure_count = failure_count + 1
        return success_dict, failure_dict


def generate_json_report(json_file_name: str = "default_name"):
    """
    :param json_file_name: save json file's name
    """
    lock = Lock()
    success_dict, failure_dict = generate_json()
    try:
        lock.acquire()
        with open(json_file_name + "_failure.json", "w+") as file_to_write:
            json.dump(dict(failure_dict), file_to_write, indent=4)
    except Exception as error:
        print(repr(error), file=sys.stderr)
    finally:
        lock.release()
    try:
        lock.acquire()
        with open(json_file_name + "_success.json", "w+") as file_to_write:
            json.dump(dict(success_dict), file_to_write, indent=4)
    except Exception as error:
        print(repr(error), file=sys.stderr)
    finally:
        lock.release()



# test api method
from je_api_testka.requests_wrapper.request_method import test_api_method
# callback
from je_api_testka.utils.callback.callback_function_executor import callback_executor
# graph
from je_api_testka.utils.create_graph.create_graph import make_tkinter_request_time_graph
from je_api_testka.utils.executor.action_executor import add_command_to_executor
# execute
from je_api_testka.utils.executor.action_executor import execute_action
from je_api_testka.utils.executor.action_executor import execute_files
from je_api_testka.utils.executor.action_executor import executor
# file process
from je_api_testka.utils.file_process.get_dir_file_list import get_dir_files_as_list
# html_output
from je_api_testka.utils.generate_report.html_report_generate import generate_html
from je_api_testka.utils.generate_report.html_report_generate import generate_html_report
from je_api_testka.utils.generate_report.json_report import generate_json
from je_api_testka.utils.generate_report.json_report import generate_json_report
from je_api_testka.utils.generate_report.xml_report import generate_xml
from je_api_testka.utils.generate_report.xml_report import generate_xml_report
from je_api_testka.utils.json.json_file.json_file import read_action_json
# json
from je_api_testka.utils.json.json_file.json_file import write_action_json
from je_api_testka.utils.json.json_format.json_process import reformat_json
# socket server
from je_api_testka.utils.socket_server.api_testka_socket_server import start_apitestka_socket_server
# test_record
from je_api_testka.utils.test_record.test_record_class import test_record_instance
from je_api_testka.utils.xml.change_xml_structure.change_xml_structure import dict_to_elements_tree
from je_api_testka.utils.xml.change_xml_structure.change_xml_structure import elements_tree_to_dict
# xml
from je_api_testka.utils.xml.xml_file.xml_file import XMLParser
from je_api_testka.utils.xml.xml_file.xml_file import reformat_xml_file

__all__ = ["test_api_method",
           "make_tkinter_request_time_graph", "add_command_to_executor",
           "execute_action", "execute_files", "executor",
           "get_dir_files_as_list",
           "generate_html", "generate_html_report", "read_action_json",
           "write_action_json", "reformat_json", "generate_json", "generate_json_report",
           "start_apitestka_socket_server",
           "test_record_instance", "dict_to_elements_tree", "elements_tree_to_dict",
           "XMLParser", "reformat_xml_file", "generate_xml", "generate_xml_report",
           "callback_executor"
           ]
